
GAME_UPDATE_SPEED = 1/60; // how much time in seconds elapses in each game frame
GAME_RENDER_SPEED = 1/30; // how fast we actually render the screen

ANIMATION_SPEED = 8 * GAME_UPDATE_SPEED;
PICKUP_ANIMATION_SPEED = 4 * GAME_UPDATE_SPEED;
TURN_ANIMATION_SPEED = 16 * GAME_UPDATE_SPEED;
PLAYER_MOVE_SPEED = 150.0; // how fast you move
INPUT_SMOOTHING = 0.2; // lower numbers smooth more, gives the movement a polished, less jerky feeling
PLAYER_COLOR = [255,115,87];
MOVE_RIPPLE_DIST = 15; // the player spawns ripples 

SPAWN_LABEL = "Spawning";
FONT = "20px Georgia";
DOT_STYLE = '#FFFFFF';
DOT_RADIUS = 20;

function timestamp() {
	return Date.now() / 1000;
}

CENTER_SPAWNER = 0
SPRINKLE_SPAWNER = 1

g_CanvasWidth = 1000;
g_CanvasHeight = 1000;

g_Waves = []; // these represent our rideable circles
g_EffectWaves = []; // these are our effect circles
g_Pulsers = []; // a 'pulser' spawns waves at intervals
g_CalculatedTime = timestamp();
g_Dots = []; // the stuff to eat
g_NextSpawnRippleDist = 0;

g_PlayerWave = null; // The current wave the player is attached to
g_PlayerAngle = 0; // The current angle the player is attached to
g_PlayerNextPulser = null; // if not null, will attach the player to this pulser's next wave. This happens when the player gets stuck
g_PlayerX = 0, g_PlayerY = 0; // player position
g_PlayerFrame = 0; // current frame of the player
g_LastPlayerX = 0, g_LastPlayerY = 0; // last player position
g_LastPlayerDirX = 0, g_LastPlayerDirY = 0; // last player measured direction
g_KillPlayer = false; // if something wants to kill the player, set this to true and it will be picked up. I do things this way to avoid bugs with having to add and remove things from the javascript arrays while I'm iterating over them.
g_PlayerInput = 0;
g_PlayerIsAlive = false;
g_PlayerTurning = false;
g_PlayerImageRotation = 0; // the angle used to draw the player image
g_PlayerInLimbo = false; // player is spinning around
g_EndGame = false; // finished all the levels
g_GenerateLevelFunc = null;
g_CurrentDotFrame = 0;
g_IsLeftDown = false, g_IsRightDown = false;
g_WaveStartWidth = 2;
g_WaveEndWidth = 20;

g_CurrentLevel = -1;

// some math utility functions
function rgba(r, g, b, a) {

	return 'rgba(' + Math.floor(r) + ','+ Math.floor(g) + ',' + Math.floor(b) + ',' + a + ')';
}

function square(x) {

	return x*x;
}

function distanceSq(x1,y1, x2,y2) {

	return square(x1-x2) + square(y1-y2);
}

// image loading support
g_ImageCache = {}
g_SoundCache = {}
g_AssetsToLoad = 0;

function onAssetLoaded() {
	g_AssetsToLoad--;
}

function loadImage(path) {

	if (path in g_ImageCache) {
		return g_ImageCache[path];
	}

	g_AssetsToLoad++;

	var image = new Image();
	image.onload = onAssetLoaded;
	image.src = path;

	g_ImageCache[path] = image;
	return image;
}

function playAudio(obj) {

	try {
		obj.audio[obj.index].play();

	} catch (err) {

	}

	obj.index = (obj.index+1)%obj.audio.length;
}

function getWaveWidth(w) {
	var a = Math.max(0, 1-w.currentRadius/w.maxRadius);

	return w.startWidth + (w.endWidth-w.startWidth)*square(1-a);
}

function loadAudio(path, count=1) {

	if (path in g_SoundCache) {
		return g_SoundCache[path];
	}

	result = new Object();
	result.audio = []
	result.index = 0;

	for (var n = 0; n < count; ++n){
		var a = new Audio(path);
		a.load();
		result.audio.push(a);
	}

	g_SoundCache[path] = result;

	return result;

}

g_DotRadiusSq = square(DOT_RADIUS);

function addEffectWave(x, y, color, speed, radius, startWidth, endWidth) {

	var w = new Object();
	w.x = x;
	w.y = y;
	w.color = color;
	w.speed = speed;
	w.currentRadius = 1;
	w.lastRadius = 1;
	w.maxRadius = radius;
	w.startWidth = startWidth;
	w.endWidth = endWidth;
	
	g_EffectWaves.push(w);

	return w;
}

function addPulseWave(pulser, x, y) {

	var w = new Object();
	w.x = x;
	w.y = y;
	w.image = pulser.image;
	w.playerImage = pulser.playerImage;
	w.speed = pulser.waveSpeed;
	w.currentRadius = 1;
	w.lastRadius = 1;
	w.maxRadius = pulser.radius;
	w.attachable = true;
	w.startWidth = pulser.startWidth;
	w.endWidth = pulser.endWidth;
	w.pulser = pulser;
	
	g_Waves.push(w);

	return w;
}

function addPulser(x, y, image, playerImage, spawnDelay, radius, waveSpeed, type, startWidth, endWidth, initialTime = 0) {

	var p = new Object();
	p.x = x;
	p.y = y;
	p.image = image;
	p.playerImage = playerImage;
	p.nextSpawnTime = initialTime;
	p.radius = radius;
	p.spawnDelay = spawnDelay;
	p.waveSpeed = waveSpeed;
	p.startWidth = startWidth;
	p.endWidth = endWidth;
	p.type = type;

	g_Pulsers.push(p);
	return p;
}


function addDot(x, y) {

	var d = new Object();
	d.x = x;
	d.y = y;
	d.frameOffset = Math.random() * 4;

	g_Dots.push(d);
	return d;
}

function updatePulsers() {

	for (var n = 0; n < g_Pulsers.length; ++n) {

		var p = g_Pulsers[n];

		p.nextSpawnTime -= GAME_UPDATE_SPEED;

		if (p.nextSpawnTime < 0) {

			var w = addPulseWave(p, p.x, p.y);

			if (g_PlayerNextPulser == p && !g_PlayerIsAlive) {
				// if the player was waiting on us to spawn, use our wave
				g_PlayerWave = w;
				g_PlayerIsAlive = true;
			}

			p.nextSpawnTime = p.spawnDelay;
		}
	}
}

function drawPulsers(ctx) {

	for (var n = 0; n < g_Pulsers.length; ++n) {
		var p = g_Pulsers[n];

		ctx.beginPath();
		ctx.fillStyle = ctx.createPattern(g_BackgroundDark, "no-repeat");

		ctx.arc(p.x, p.y, p.radius * 0.9, 0, Math.PI*2, true);
		ctx.fill();
	}

	if (g_PlayerNextPulser) {
		ctx.drawImage(g_PulserImage, g_PlayerNextPulser.x-g_PulserImage.width/2, g_PlayerNextPulser.y-g_PulserImage.height/2);
	}
}

function updateWaves(list) {

	for (var n = 0; n < list.length;) {

		var w = list[n];
		w.lastRadius = w.currentRadius;
		w.currentRadius += w.speed * GAME_UPDATE_SPEED;

		if (w.currentRadius >= w.maxRadius) {

			list.splice(n, 1);

			if (g_PlayerWave == w) {
				// detach from the wave
				g_PlayerWave = null;
			}

			continue;
		}

		++n;
	}
}

function drawRidableWaves(ctx) {

	for (var n = 0; n < g_Waves.length; ++n) {
		var w = g_Waves[n];

		ctx.beginPath();

		var a = Math.max(0, 1-w.currentRadius/w.maxRadius);

		var use_img = (w == g_PlayerWave) ? w.playerImage : w.image;

		ctx.strokeStyle = ctx.createPattern(use_img, "no-repeat");

		ctx.lineWidth = getWaveWidth(w);

		ctx.translate(w.x-use_img.width/2, w.y-use_img.height/2);

		ctx.arc(use_img.width/2, use_img.height/2, w.currentRadius+ctx.lineWidth/2, 0, Math.PI*2, true);
		ctx.stroke();

		ctx.setTransform(1,0,0,1,0,0);
	}
}

function drawEffectWaves(ctx) {

	for (var n = 0; n < g_EffectWaves.length; ++n) {
		var w = g_EffectWaves[n];

		ctx.beginPath();

		var a = 1-w.currentRadius/w.maxRadius;
		ctx.strokeStyle = rgba(w.color[0], w.color[1], w.color[2], a);

		ctx.lineWidth = w.startWidth + (w.endWidth-w.startWidth)*square(1-a);

		ctx.arc(w.x, w.y, w.currentRadius, 0, Math.PI*2, true);
		ctx.stroke();
	}
}


function onDotCollected(d) {

	addEffectWave(d.x, d.y, [255,255,255], 5, 10, 2, 4);
	playAudio(g_BubbleCollectedSound);
}

function updateDots() {

	g_CurrentDotFrame += PICKUP_ANIMATION_SPEED;
	while (g_CurrentDotFrame > g_DotImages.length) {
		g_CurrentDotFrame -= g_DotImages.length;
	}

	var pdx = g_PlayerX-g_LastPlayerX;
	var pdy = g_PlayerY-g_LastPlayerY;

	for (var n = 0; n < g_Dots.length;) {

		var d = g_Dots[n];

		// we want to test the distance to the line created from the previous player position
		// to the new one.

		var distanceAlongLine = Math.max(0, Math.min(1, (d.x*pdx)+(d.y*pdy)));

		// the dot is how far along pdx we are
		var pointOnLineX = g_LastPlayerX + distanceAlongLine * pdx;
		var pointOnLineY = g_LastPlayerY + distanceAlongLine * pdy;

		var dist_to_dot = distanceSq(pointOnLineX, pointOnLineY, d.x, d.y);

		if (dist_to_dot < g_DotRadiusSq) {

			onDotCollected(d);
			g_Dots.splice(n, 1);
			continue;
		}

		++n;
	}
}

function drawDots(ctx) {
	
	for (var n = 0; n < g_Dots.length; ++n) {

		var d = g_Dots[n];
		var img = g_DotImages[Math.floor(g_CurrentDotFrame+d.frameOffset)%g_DotImages.length];

		ctx.drawImage(img, d.x-img.width/2, d.y-img.height/2);
	}
}

function addRandomDots(count) {

	if (g_Pulsers.length == 0)
		return;

	var cvs = document.getElementById("cvs");

	for (var n = 0; n < count; ++n) {

		// must be placed in the radius of a pulser or it will not be collectible
		var pindex = Math.floor(Math.random()*g_Pulsers.length);
		var pulser = g_Pulsers[pindex];

		while (true) {

			var px = Math.random()*g_CanvasWidth*0.95 + g_CanvasWidth * 0.025;;
			var py = Math.random()*g_CanvasHeight*0.95 + g_CanvasHeight * 0.025;;

			if (distanceSq(px, py, pulser.x, pulser.y) < square(pulser.radius)) {

				addDot(px, py);
				break;
			}
		}
	}
}

function updatePlayer(ctx) {

	g_LastPlayerX = g_PlayerX;
	g_LastPlayerY = g_PlayerY;

	var was_in_limbo = g_PlayerInLimbo;

	// this is the input for the player
	var moveSpeed = PLAYER_MOVE_SPEED * GAME_UPDATE_SPEED;
	var input = (g_IsLeftDown ? moveSpeed : 0) + (g_IsRightDown ? -moveSpeed : 0);

	if (g_PlayerWave == null) {

		// if we're not waiting to respawn, and not attached to a wave,
		// we are 'stranded', attach to the next wave that passes by
		// this is fairly complex, and requires looking at all of the current waves,
		// and if we are outside the radius last frame, and inside the radius this frame
		// then we attach

		if (!g_KillPlayer) {

			for (var n = 0; n < g_Waves.length; ++n) {

				var w = g_Waves[n];

				if (!w.attachable)
					continue;

				var oldRadiusSq = square(w.lastRadius);
				var newRadiusSq = square(w.currentRadius);

				// use pythagoras to find the square distance to the wave
				var ox = g_PlayerX-w.x;
				var oy = g_PlayerY-w.y;
				var dsq = square(ox) + square(oy);

				// I compare the squared value because the square root function is slower than two extra multiplies
				if (oldRadiusSq < dsq && newRadiusSq >= dsq) {
					// found a wave to attach to
					g_PlayerWave = w;
					g_PlayerNextPulser = w.pulser;

					// find the angle we are to this new circle
					g_PlayerAngle = Math.atan2(ox, oy);

					if (g_PlayerIsAlive) {
						playAudio(g_DolphinAttachToRingSound);
					}
					break;
				}
			}
		}
	}
	else {

		var radius = g_PlayerWave.currentRadius + getWaveWidth(g_PlayerWave)/2;

		if (Math.abs(input - g_PlayerInput) > moveSpeed) { 
			// tests for a turnaround time, works because our input is smoothed, so if you are headed in one direction
			// and immediately head in the other, the difference between our smoothed input and our desired input
			// will be over our move speed

			if (!g_PlayerTurning){

				g_PlayerTurning = true;
				g_PlayerFrame = g_PlayerTurnFrame;
			}
		}

		// interpolate to the desired input to just smooth things out
		g_PlayerInput += (input-g_PlayerInput) * INPUT_SMOOTHING;
		g_PlayerAngle += g_PlayerInput / Math.max(1,radius);

		// uses math to force the player position at this angle
		g_PlayerX = g_PlayerWave.x + Math.sin(g_PlayerAngle) * radius;
		g_PlayerY = g_PlayerWave.y + Math.cos(g_PlayerAngle) * radius;

		if (g_PlayerX < 0 || g_PlayerY < 0 || g_PlayerX >= g_CanvasWidth|| g_PlayerY >= g_CanvasHeight) {

			g_KillPlayer = true;
		}
	}

	if (g_KillPlayer) {

		playAudio(g_DolphinNoiseResetSound);

		g_KillPlayer = false;

		g_PlayerWave = null;
		g_PlayerInput = 0;
		g_PlayerIsAlive = false;

		// play a little effect here with the waves
		addEffectWave(g_PlayerX, g_PlayerY, PLAYER_COLOR, 10, 20, 4, 0.1);
	}

	// update our animation
	if (g_PlayerIsAlive) {	

		g_PlayerFrame += g_PlayerTurning ? TURN_ANIMATION_SPEED : ANIMATION_SPEED;

		// loop the frame
		if (g_PlayerFrame > g_PlayerTurnFrame) {

			if (g_PlayerTurning) {

				if (g_PlayerFrame > g_PlayerFrames.length) {
					g_PlayerTurning = false;
					g_PlayerFrame -= g_PlayerFrames.length;
				}
			}
			else {

				g_PlayerFrame -= g_PlayerTurnFrame;
			}
		}

		var ox = g_PlayerX - g_LastPlayerX;
		var oy = g_PlayerY - g_LastPlayerY;

		if (ox != 0 || oy != 0) {

			if (Math.abs(input) > 0) {
				g_NextSpawnRippleDist -= Math.sqrt(square(ox)+square(oy));
				if (g_NextSpawnRippleDist < 0) {
					g_NextSpawnRippleDist = Math.max(0, g_NextSpawnRippleDist+MOVE_RIPPLE_DIST);

					addEffectWave(g_PlayerX, g_PlayerY, [255,255,255], 5, 10, 2, 4);
				}
			}

			g_PlayerImageRotation = Math.atan2(oy, ox) + Math.PI/2;
			g_PlayerInLimbo = false;

		}
		else {

			g_PlayerInLimbo = true;
			g_PlayerImageRotation += 0.05;
		}

		g_LastPlayerDirX = ox;
		g_LastPlayerDirY = oy;

		if (g_PlayerInLimbo ) {

			var in_range = false;

			// kill the player if we aren't in a pulse range
			for (var n = 0; n < g_Pulsers.length; ++n) {

				var p = g_Pulsers[n];

				if (distanceSq(g_PlayerX, g_PlayerY, p.x, p.y) < square(p.radius*0.99)) {
					
					in_range = true;
				}
			}

			if (!in_range) {

				g_KillPlayer = true;
			} else if (!was_in_limbo) {

				playAudio(g_DeatchFromRingSound);

			}
		}
	}
}

function drawPlayer(ctx) {

	if (g_PlayerIsAlive) {

		var useFrame = Math.floor(g_PlayerInLimbo ? 7 : g_PlayerFrame);

		var img = g_PlayerFrames[useFrame];

		var use_rotation = g_PlayerImageRotation + (useFrame >= g_PlayerTurnFrame ? Math.PI : 0);		

		ctx.translate(g_PlayerX, g_PlayerY);
		ctx.rotate(use_rotation);
		ctx.drawImage(img, -img.width/2, -img.height/2);
	}

	// reset transform
	ctx.setTransform(1,0,0,1,0,0);
}

function addRandomPulser(min_x, min_y, max_x, max_y, image_idx, type, startWidth, endWidth) {
	var px = 0, py = 0, check_point = true;

	image_idx = image_idx%g_RippleImages.length;
	var radius = g_RippleImages[image_idx].width/2;

	var attempts = 0;

	while (check_point && attempts < 100) {

		attempts++;

		px = Math.random()*(max_x-min_x)+min_x;
		py = Math.random()*(max_y-min_y)+min_y;

		if (g_Pulsers.length > 0) {

			var too_close = false;
			var connected = false;

			for (var n = 0; n < g_Pulsers.length; ++n) {

				var p = g_Pulsers[n];

				var distSq = distanceSq(p.x, p.y, px, py);

				if (distSq < square((p.radius+radius)*0.25)){

					too_close = true;
				} 

				if (distSq < square((p.radius+radius)*0.75)) {

					connected = true;
				}
			}

			if (!too_close && connected) {

				check_point = false;
			}

		} else {

			check_point = false;
		}
	}

	var p = addPulser(
		px,
		py,
		g_RippleImages[image_idx],
		g_PlayerRippleImages[image_idx], 
		Math.random() + 2, // spawn delay
		radius, // radius 
		Math.random()*20 + 20,
		type,
		startWidth,
		endWidth); // wave speed

	if (g_PlayerNextPulser == null) {

		g_PlayerNextPulser = p;
	}
}

function generateLevel(level) {

	g_Pulsers = []
	g_Waves = []
	g_PlayerNextPulser = null;
	g_PlayerWave = null;

	if (level > 0)
		g_KillPlayer = true;

	for (var n = 0; n < level+1; ++n) {

		var level_bounds = [
			[0.2, 0.2, 0.5, 0.5],
			[0.5, 0.5, 0.8, 0.8],
			[0.5, 0.2, 0.8, 0.5],
			[0.2, 0.5, 0.5, 0.8],
			[0.2, 0.2, 0.8, 0.8],
			[0.2, 0.2, 0.8, 0.8],
		];

		addRandomPulser(g_CanvasWidth*level_bounds[n][0], 
			g_CanvasHeight*level_bounds[n][1],
			g_CanvasWidth*level_bounds[n][2],
			g_CanvasHeight*level_bounds[n][3],
			n, CENTER_SPAWNER, g_WaveStartWidth, g_WaveEndWidth);
	}

	addRandomDots(2 + level * 5);
}


function update() {

	if (g_Dots.length == 0 && !g_EndGame && g_GenerateLevelFunc == null) { // means we cleared a level

		g_CurrentLevel++;
		if (g_CurrentLevel == 6) {
			g_EndGame = true;
			return;
		}

		var stage_idx = Math.floor(g_CurrentLevel % g_Levels.length);
		var stage = g_Levels[stage_idx]; // defined in levels.js

		// load all of our art assets
		g_BackgroundImage = loadImage(stage.background_art);
		g_BackgroundDark = loadImage(stage.background_mask_art);
		g_RippleImages = [
			loadImage(stage.circle_art[0]),
			loadImage(stage.circle_art[1]),
			loadImage(stage.circle_art[2]),
			loadImage(stage.circle_art[3]),
		];
		g_PlayerRippleImages = [
			loadImage(stage.circle_art_orange[0]),
			loadImage(stage.circle_art_orange[1]),
			loadImage(stage.circle_art_orange[2]),
			loadImage(stage.circle_art_orange[3]),
		];
		g_OverlayImage = stage.overlay_art != null ? loadImage(stage.overlay_art) : null;

		g_PulserImage = loadImage(stage.spawning_station_art);
		g_DotImages = [];

		for (var n = 0; n < stage.pickup_art.length; ++n) {
			g_DotImages.push(loadImage(stage.pickup_art[n]))
		}

		g_WaveStartWidth = stage.start_width;
		g_WaveEndWidth = stage.end_width;

		g_GenerateLevelFunc = generateLevel;
	}

	if (g_AssetsToLoad > 0 || g_EndGame) {
		
		return;
	}

	if (g_GenerateLevelFunc) {
		g_GenerateLevelFunc(g_CurrentLevel);
		playAudio(g_LevelStartSound);
		g_GenerateLevelFunc = null;
	}

	// this clears the background
	updatePulsers();
	updateWaves(g_Waves);
	updateWaves(g_EffectWaves);
	updatePlayer();
	updateDots();
}

function drawLoadScreen() {

	var cvs = document.getElementById("cvs");
	var ctx = cvs.getContext('2d');

	ctx.fillStyle = '#000';
	ctx.fillRect(0, 0, g_CanvasWidth, g_CanvasHeight);

	ctx.fillStyle = '#FFF';
	ctx.font = '48px Pacifico serif';
	var label = "Loading";

	var w = ctx.measureText(label).width;

	ctx.fillText(label, (cvs.width-w)/2, cvs.height/2+10);
}

function drawEndGame() {

	var cvs = document.getElementById("cvs");
	var ctx = cvs.getContext('2d');

	ctx.drawImage(g_WinScreen, 0, 0);
}

function draw() {

	var cvs = document.getElementById("cvs");

	var ctx = cvs.getContext('2d');

	ctx.drawImage(g_BackgroundImage,0,0);

	drawPulsers(ctx);
	drawRidableWaves(ctx);
	drawEffectWaves(ctx);
	drawPlayer(ctx);
	drawDots(ctx);

	if (g_OverlayImage != null) {
		ctx.drawImage(g_OverlayImage,0,0);
	}
}

function mainLoop() {

	// this updates our fixed game loop multiple times until we reach real time
	while (g_CalculatedTime < timestamp()) {
		
		update();
		g_CalculatedTime += GAME_UPDATE_SPEED;
	}

	if (g_AssetsToLoad > 0) {

		drawLoadScreen();

	} else if (g_EndGame) {

		drawEndGame();

	} else {

		draw();
	}
}

LEFT_KEYCODE = 37;
RIGHT_KEYCODE = 39;
SPACE_KEYCODE = 32;
UP_KEYCODE = 38;
DOWN_KEYCODE = 40;

// handles input
document.addEventListener('keydown', function(e) {
	
	if (e.keyCode == LEFT_KEYCODE) {

		e.preventDefault();
		g_IsLeftDown = true;

	} else if (e.keyCode == RIGHT_KEYCODE) {
		
		e.preventDefault();
		g_IsRightDown = true;

	} else if (e.keyCode == SPACE_KEYCODE) {

		e.preventDefault();
		
		if (g_PlayerWave) {

			playAudio(g_DeatchFromRingSound);
			
			g_PlayerWave.attachable = false;
			g_PlayerWave = null;
		}
	} else if (e.keyCode == UP_KEYCODE || e.keyCode == DOWN_KEYCODE) {

		e.preventDefault();

	}
});

document.addEventListener('keyup', function(e) {
	
	if (e.keyCode == LEFT_KEYCODE) {
	
		e.preventDefault();
		g_IsLeftDown = false;

	} else if (e.keyCode == RIGHT_KEYCODE) {
	
		e.preventDefault();
		g_IsRightDown = false;

	} else if (e.keyCode == SPACE_KEYCODE || e.keyCode == UP_KEYCODE || e.keyCode == DOWN_KEYCODE) {

		e.preventDefault();
	}
});

//function addPulser(x, y, color, spawnDelay, radius, waveSpeed, initialTime = 0) {
window.onload = function(e) {

	var cvs = document.getElementById("cvs");
	g_CanvasWidth = cvs.width;
	g_CanvasHeight = cvs.height;

	// load some images
	g_PlayerFrames = [
		loadImage('odolphin2_01_v.png'), 
		loadImage('odolphin2_00_v.png'), 
		loadImage('odolphin2_02_v.png'), 
		loadImage('odolphin2_00_v.png'),
		loadImage('odolphin2_07_v.png'), 
		loadImage('odolphin2_08_v.png'),
		loadImage('odolphin2_04_v.png'),
		loadImage('odolphin2_05_v.png'),		
		loadImage('odolphin2_06_v.png'),		
		loadImage('odolphin2_09_v.png'),		
		loadImage('odolphin2_10_v.png'),		
		loadImage('odolphin2_11_v.png'),		
	];

	g_PlayerTurnFrame = 4; // if we aren't turning we loop back to beginning once we hit this

	g_LevelStartSound = loadAudio('level_start.mp3', 2);
	g_DolphinAttachToRingSound = loadAudio('dolphin_flip.mp3', 5)
	g_DolphinNoiseResetSound = loadAudio('dolphin_reset.mp3');
	g_DeatchFromRingSound = loadAudio('dolphin_stuck.mp3', 2);
	g_BubbleCollectedSound = loadAudio('bubble_collected.mp3', 10);

	g_WinScreen = loadImage("youwin_screen.jpg");

	setInterval(mainLoop, GAME_RENDER_SPEED);
};

//	addPulser(400,400, [255,0,127], 0.5, 150, 1.2);

